"""
author comger@gmail.com
统一异常处理机制
"""
import logging
import traceback
from cms.core import config
from starlette.responses import HTMLResponse


log = logging.getLogger()


class ErrorHandler:

    @classmethod
    def handler(cls, e: Exception, ttype, tvalue, ttraceback):

        if config.DEBUG:
            log.error(e)
            content_html = f"<p>{ttype}</p>"
            for info in traceback.format_tb(ttraceback):
                content_html += f"<p>{info}</p>"

            html = f"<html><body><h2>{tvalue}</h2>{content_html}</body></html>"
            return HTMLResponse(content=html)
        else:
            msg = cls.get_error(ttype, e=e)
            return HTMLResponse(content=f"{msg},请联系开发人员")

    @classmethod
    def get_error(cls, ttype, e=None):
        name = ttype.__name__

        if name.endswith("DoesNotExist"):
            return "你输入的资源不存在"
        elif name.endswith("UniqueViolation"):
            return "你输入的资源已存在"
        elif name.endswith("IntegrityError"):
            return e.__str__()
        elif name.endswith("ForeignKeyViolation"):
            return "你输入的外键ID不存在"
        else:
            return "未知异常"



import os


try:
    from pathlib import Path
    from dotenv import load_dotenv
    for p in Path("../").rglob('*dev.env'):
        load_dotenv(dotenv_path=p)
except Exception:
    pass



def getenv_boolean(var_name, default_value=False):
    result = default_value
    env_value = os.getenv(var_name)
    if env_value is not None:
        result = env_value.upper() in ("TRUE", "1")
    return result


DEBUG = getenv_boolean("DEBUG", True)
API_V1_STR = "/api/v1"
LOGIN_API = os.getenv('LOGIN_API', '/api/v1/login/access-token')
LOG_MAX_BYTES = int(os.getenv("LOG_MAX_BYTES", 1024 * 1024 * 100))
LOG_MAX_NUM = int(os.getenv("LOG_MAX_NUM", 8))

ALGORITHM = os.getenv("ALGORITHM", 'HS256')
SECRET_KEY = os.getenv("SECRET_KEY", "smartbow")

# 60 minutes * 24 hours * 8 days = 8 days
ACCESS_TOKEN_EXPIRE_MINUTES = 60 * 24 * 8
SERVER_NAME = os.getenv("SERVER_NAME", 'localhost')
SERVER_HOST = os.getenv("SERVER_HOST")
BACKEND_CORS_ORIGINS = os.getenv("BACKEND_CORS_ORIGINS", "*")
PROJECT_NAME = os.getenv("PROJECT_NAME", "Web Server")

POSTGRES_SERVER = os.getenv("POSTGRES_SERVER", "192.168.111.147")
POSTGRES_PORT = os.getenv("POSTGRES_PORT", "5432")
POSTGRES_USER = os.getenv("POSTGRES_USER", "postgres")
POSTGRES_PASSWORD = os.getenv("POSTGRES_PASSWORD", '123456')
POSTGRES_DB = os.getenv("POSTGRES_DB", 'democms')


FIRST_USER = os.getenv("FIRST_USER", "admin")
FIRST_USER_PASSWORD = os.getenv("FIRST_USER_PASSWORD", "admin")


CLIENT_ID = os.getenv("CLIENT_ID", "17da3dda-f3c6-11e9-b4d1-0242ac170002")
CLIENT_SECRET = os.getenv("CLIENT_SECRET", "17dce0d0-f3c6-11e9-b4d1-0242ac170002")
USERCENTER_HOST = os.getenv("USERCENTER_HOST", "http://ucapi.sb2.org")



"""
author comger@gmail.com
统一日志处理
* 标准日志输出
    * Debug 模式下输出为控制台
    * 非Debug 模式下输出为文件及控制台
    * 日志文件可以自动按日志大小切分
"""
import os
import logging
import logging.handlers
from cms.core import config


# 最大日志保留数
B_COUNT = 8

# 日志等级
LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}


class Logger:
    """
    封装logging类
    >>level 可以输出的最低级别
    >>type    0:输出到前台和文件 1:输出到文件
    """

    def __init__(self, name='output', level=logging.DEBUG, type=0):
        path = os.getcwd() + "/logs"
        if not os.path.exists(path):
            os.mkdir(path)

        log_file = f"{path}/{name}.log"
        self._log = logging.getLogger(log_file)

        self._log.setLevel(level)
        lfh = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=config.LOG_MAX_BYTES, backupCount=config.LOG_MAX_NUM)

        formatter = logging.Formatter(
            "%(asctime)s - %(levelname)s - %(message)s")

        lfh.setFormatter(formatter)

        self._log.addHandler(lfh)

        if type == 0:
            ch = logging.StreamHandler()
            ch.setFormatter(formatter)
            self._log.addHandler(ch)

    def getLogger(self):
        return self._log


logger = Logger().getLogger()


from typing import Any, Dict, Generic, List, Optional, Type, TypeVar, Union

from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel, Field
from cms.core.db import DataViewsionManage, DBBaseModel


ModelType = TypeVar("ModelType", bound=DBBaseModel)
CreateSchemaType = TypeVar("CreateSchemaType", bound=BaseModel)
UpdateSchemaType = TypeVar("UpdateSchemaType", bound=BaseModel)
OutSchemaType = TypeVar("OutSchemaType", bound=BaseModel)

class PageInfo(BaseModel):
    total: int = Field(..., title='总数')
    data: List[OutSchemaType] = Field([], title='列表')


class CRUDBase(Generic[ModelType, CreateSchemaType, UpdateSchemaType, OutSchemaType]):
    def __init__(self, model: Type[ModelType]):
        """
        CRUD object with default methods to Create, Read, Update, Delete (CRUD).
        **Parameters**
        * `model`: A SQLAlchemy model class
        * `schema`: A Pydantic model (schema) class
        """
        self.model = model

    async def get(self, pgdvm: DataViewsionManage, id: Any) -> Optional[ModelType]:
        obj = await pgdvm.get(self.model, id=id)
        return obj

    async def get_multi(
        self, pgdvm: DataViewsionManage, *, skip: int = 0, limit: int = 100, **kwargs
    ) -> List[ModelType]:
        query = self.model.select().offset(skip).limit(limit)
        lst = await pgdvm.execute(query)
        return lst

    async def get_page(
        self, pgdvm: DataViewsionManage, *, page_index: int = 1, page_size: int = 100, **kwargs
    ) -> (int, List[ModelType]):
        count = await self.count(pgdvm=pgdvm)
        query = self.model.select().offset(page_size * (page_index - 1)).limit(page_size)
        lst = await pgdvm.execute(query)
        return count, lst


    async def count(self, pgdvm: DataViewsionManage, **kwargs) -> int:
        query = self.model.select()
        return await pgdvm.count(query)

    async def create(self, pgdvm: DataViewsionManage, *, obj_in: CreateSchemaType) -> ModelType:
        obj_in_data = jsonable_encoder(obj_in)
        obj = await pgdvm.create(self.model, **obj_in_data)
        return obj

    async def update(
        self,
        pgdvm: DataViewsionManage,
        *,
        db_obj: ModelType,
        obj_in: Union[UpdateSchemaType, Dict[str, Any]]
    ) -> ModelType:
        if isinstance(obj_in, dict):
            update_data = obj_in
        else:
            update_data = obj_in.dict(exclude_unset=True)

        for field, val in update_data.items():
            setattr(db_obj, field, val)

        await pgdvm.update(db_obj)
        return db_obj

    async def remove(self, pgdvm: DataViewsionManage, *, id: int) -> ModelType:
        obj = await pgdvm.get(self.model, id=id)
        await pgdvm.delete(obj)
        return obj



from typing import List
from fastapi import APIRouter, Depends, HTTPException, Query, Path
from fastapi.security import OAuth2PasswordRequestForm

from cms.core.auth import create_access_token, get_current_user
from cms.core.db import depends_db, m2d
from cms.core.logger import logger
from cms.logic.common.user import crud_user, Token, UserCreate, UserUpdate, UserUpdateMe, UserOut
from cms.logic.common.user import crud_role, RoleCreate, RoleUpdate


router = APIRouter()


@router.post("/login/access-token", response_model=Token, summary='用户登录')
async def login_access_token(
    form_data: OAuth2PasswordRequestForm = Depends(),
    pgdb=Depends(depends_db)
):
    """
    OAuth2 密码模式登录
    """
    status, user = await crud_user.login(pgdvm=pgdb, username=form_data.username, password=form_data.password)
    if not status:
        raise HTTPException(status_code=403, detail=user)

    data = {"id": user.id,
            "uid": user.uid,
            "username": user.username,
            "nickname": user.nickname,
            "email": user.email,
            "is_superuser": user.is_superuser,
            "is_active": user.is_active,
            "role_id": user.role_id,
            "attrs": user.attrs}

    return {
        "access_token": create_access_token(data=data),
        "token_type": "bearer"
    }


@router.get("/users", summary='获取用户列表', response_model=List[UserOut])
async def read_users(
    query: str = Query(None, title='搜索关键字',
                       description='模糊查询匹配(用户名, 昵称, 邮箱和电话)'),
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user),
):
    lst = await crud_user.get_multi(pgdvm=pgdb, querykey=query)
    return m2d(lst)


@router.get('/users/me', summary='获取当前登录用户信息', response_model=UserOut)
async def user_me(
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user)
):
    """
    获取当前登录用户信息
    """
    try:
        user = await crud_user.get(pgdvm=pgdb, id=current_user['id'])
        return m2d(user)
    except Exception as e:
        logger.error(e)


@router.put('/users/me', summary='修改当前登录用户信息')
async def update_user_me(
    user_in: UserUpdateMe,
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user)
):
    """
    获取当前登录用户信息
    """
    user = await crud_user.get(pgdvm=pgdb, id=current_user['id'])
    user = await crud_user.update(pgdvm=pgdb, db_obj=user, obj_in=user_in)
    return m2d(user)


@router.get('/users/{id}', summary='按ID获取信息', response_model=UserOut)
async def user_one(
    id: int = Path(..., title="ID"),
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user)
):
    """
    按ID获取信息
    """
    user = await crud_user.get(pgdvm=pgdb, id=id)
    logger.info(user.username)
    logger.info(f'role:{user.role_id}')
    return m2d(user)


@router.post("/users", summary='添加用户', status_code=201,  response_model=UserOut)
async def create_user(
        user_in: UserCreate,
        pgdb=Depends(depends_db),
        current_user=Depends(get_current_user)):
    """
    添加用户
    """
    user = await crud_user.create(pgdvm=pgdb, obj_in=user_in)
    return m2d(user)


@router.put("/users/{id}", summary='编辑用户', status_code=201, response_model=UserOut)
async def update_user(
        user_in: UserUpdate,
        id: int = Path(..., title="ID"),
        pgdb=Depends(depends_db),
        current_user=Depends(get_current_user)):
    """
    编辑用户, 覆盖编辑
    """
    user = await crud_user.get(pgdvm=pgdb, id=id)
    user = await crud_user.update(pgdvm=pgdb, db_obj=user, obj_in=user_in)
    return m2d(user)


@router.delete('/users/{id}', summary='按ID禁用信息')
async def del_user(
    id: int = Path(..., title="ID"),
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user)
):
    """
    按ID禁用信息
    """
    user = await crud_user.get(pgdvm=pgdb, id=id)
    user_in = UserUpdate(is_active=False)
    user = await crud_user.update(pgdvm=pgdb, db_obj=user, obj_in=user_in)
    return m2d(user)


@router.get("/roles", summary='获取角色列表')
async def read_roles(
    limit: int = Query(100, title="返回数量"),
    skip: int = Query(0, title="跳过数量"),
    pgdb=Depends(depends_db),
    current_user=Depends(get_current_user),
):
    lst = await crud_role.get_multi(pgdvm=pgdb, skip=skip, limit=limit)
    return m2d(lst)


@router.post("/roles", summary='添加角色', status_code=201)
async def create_role(
        obj_in: RoleCreate,
        pgdb=Depends(depends_db),
        current_user=Depends(get_current_user)):
    """
    添加角色
    """
    obj = await crud_role.create(pgdvm=pgdb, obj_in=obj_in)
    return m2d(obj)


@router.put("/roles/{id}", summary='编辑角色', status_code=201)
async def update_role(
        obj_in: RoleUpdate,
        id: int = Path(..., title="ID"),
        pgdb=Depends(depends_db),
        current_user=Depends(get_current_user)):
    """
    编辑角色, 覆盖编辑
    """
    obj = await crud_role.get(pgdvm=pgdb, id=id)
    obj = await crud_role.update(pgdvm=pgdb, db_obj=obj, obj_in=obj_in)
    return m2d(obj)


@router.delete("/roles/{id}", summary='删除角色', status_code=201)
async def del_role(
        id: int = Path(..., title="ID"),
        pgdb=Depends(depends_db),
        current_user=Depends(get_current_user)):
    """
    删除角色
    """
    await crud_role.remove(pgdvm=pgdb, id=id)
    return True



from fastapi import APIRouter
from cms.logic.common.crud import CRUDBase


async def list():
    # 如何知道，当前Model
    pass


async def func():
    pass


class URiRouter(APIRouter):
    def fetch(self, model: CRUDBase):
        """
        通过数据表操作类，自动添加CRUD HTTP Api
        """
        response_model = None
        tags = []
        dependencies = []
        summary = "列表查询"
        description = "列表查询、支持limit, skip"
        response_description = None
        responses = None
        methods = ["GET"]
        name = "list_objs"
        callbacks = []
        self.add_api_route(
            "/items",
            func,
            response_model=response_model,
            status_code=201,
            tags=tags or [],
            dependencies=dependencies,
            summary=summary,
            description=description,
            response_description=response_description,
            responses=responses or {},
            methods=methods,
            name=name,
            callbacks=callbacks,
        )



import asyncio
import peewee_async
import peewee_asyncext
import psycopg2

from datetime import datetime
from playhouse.shortcuts import model_to_dict
from starlette.requests import Request
from peewee import DateTimeField, SQL, Query
from playhouse.postgres_ext import Model
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT

from cms.core import config
from cms.core.logger import Logger

logger = Logger().getLogger()


def create_database():
    ''' 创建数据库实例 '''
    conn = psycopg2.connect(database="postgres",
                            port=config.POSTGRES_PORT,
                            host=config.POSTGRES_SERVER,
                            user=config.POSTGRES_USER,
                            password=config.POSTGRES_PASSWORD)
    conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    cursor = conn.cursor()
    CREATE_DB = "CREATE DATABASE {}".format(config.POSTGRES_DB)
    try:
        cursor.execute(CREATE_DB)
    except psycopg2.errors.DuplicateDatabase:
        logger.info('psycopg2.errors.DuplicateDatabase,{} is exist'.format(
            config.POSTGRES_DB))
    except psycopg2.errors.UniqueViolation:
        logger.info('psycopg2.errors.UniqueViolation,{} is exist'.format(
            config.POSTGRES_DB))
    finally:
        cursor.close()
        conn.close()


def create_hstore_extension():
    conn = psycopg2.connect(dbname=config.POSTGRES_DB,
                            port=config.POSTGRES_PORT,
                            host=config.POSTGRES_SERVER,
                            user=config.POSTGRES_USER,
                            password=config.POSTGRES_PASSWORD)
    conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    cursor = conn.cursor()
    CREATE_DB = f"CREATE EXTENSION hstore"
    try:
        cursor.execute(CREATE_DB)
    except Exception as e:
        logger.info(e)
    finally:
        cursor.close()
        conn.close()


create_database()
create_hstore_extension()

pgdb = peewee_asyncext.PooledPostgresqlExtDatabase(
    config.POSTGRES_DB,
    host=config.POSTGRES_SERVER,
    port=config.POSTGRES_PORT,
    user=config.POSTGRES_USER,
    password=config.POSTGRES_PASSWORD,
    register_hstore=True,
    max_connections=10,
)


class DataViewsionManage(peewee_async.Manager):
    async def create(self, obj, **data):
        # 生成创建ID, XX,XX 字段记录
        with pgdb.atomic():
            newObj = await super().create(obj, **data)
            return newObj

    async def update(self, obj, only=None):
        # 生成修改ID, XX,XX 字段的记录
        with pgdb.atomic():
            obj.updated_at = datetime.now()
            return await super().update(obj, only=only)

    async def delete(self,
                     obj,
                     recursive=False,
                     delete_nullable=False,
                     remove=False):
        # 生成删除ID 的记录
        with pgdb.atomic():
            if remove:
                return await super().delete(obj)
            obj.updated_at = datetime.now()
            obj.deleted_at = datetime.now()
            return await super().update(obj)

    async def execute_notdel(self, query, deleted_at=True):
        """Execute query asyncronously.
        """
        if deleted_at:
            query = query.where(SQL('deleted_at is null'))
        query = self._swap_database(query)
        return (await super().execute(query))

    async def count_notdel(self, query, deleted_at=True):
        """Perform *COUNT* aggregated query asynchronously.
        :return: number of objects in ``select()`` query
        """
        if deleted_at:
            query = query.where(SQL('deleted_at is null'))
        query = self._swap_database(query)
        return (await super().count(query, clear_limit=False))

    async def get_notdel(self, source_, *args, **kwargs):
        """Get the model instance.
        :param source_: model or base query for lookup
        Example::
            async def my_async_func():
                obj1 = await objects.get(MyModel, id=1)
                obj2 = await objects.get(MyModel, MyModel.id==1)
                obj3 = await objects.get(MyModel.select().where(MyModel.id==1))
        All will return `MyModel` instance with `id = 1`
        """
        await super().connect()

        if isinstance(source_, Query):
            query = source_
            model = query.model
        else:
            query = source_.select()
            model = source_

        conditions = list(args) + [(getattr(model, k) == v)
                                   for k, v in kwargs.items()]

        if conditions:
            query = query.where(*conditions)

        try:
            result = await self.execute_notdel(query)
            return list(result)[0]
        except IndexError:
            raise model.DoesNotExist


def get_session():
    """ 返回async peewee PostgresqlExtDatabase Manager 的"""
    loop = asyncio.get_event_loop()
    return DataViewsionManage(pgdb, loop=loop)


def depends_db(request: Request):
    """ 获取当前请求的上下文的postgresql db 异步管理实现 """
    return request.state.pgdb


class DBBaseModel(Model):
    class Meta:
        database = pgdb

    created_at = DateTimeField(default=datetime.now)
    updated_at = DateTimeField(default=datetime.now)
    deleted_at = DateTimeField(null=True)


def m2d(obj):
    """ peewee model 实现转dict 方法"""
    if type(obj) == peewee_async.AsyncQueryWrapper:
        return [model_to_dict(o, recurse=True) for o in obj]
    else:
        vals = model_to_dict(obj, recurse=True)
        for key, val in vals.items():
            if type(val) == bytes:
                vals[key] = str(val, encoding="utf8")
        return vals



"""
FastApiCli
author comger@gmail.com

fastapi_api.py -m Project -n 中文名称 -o cms所在跨径
建议在项目 main.py 文件同级建议
"""
import sys
import os
import shutil
import argparse
from tornado import template
import fastapicli


def init_logic(model, output=""):
    """ 生成逻辑代码"""
    loader = template.Loader('.')
    source_path = f"{fastapicli.__path__[0]}/logic.pyt"
    pyfbody = loader.load(source_path).generate(Model=model)
    # 保存位置
    path = f"{output}cms/logic/{str.lower(model)}.py"
    print("success created at", path)
    f = open(path, "w")
    f.write(str(pyfbody, encoding='utf-8'))
    f.close()

def init_api(model, name, output=""):
    """ 生成接口代码"""
    loader = template.Loader('.')
    source_path = f"{fastapicli.__path__[0]}/api.pyt"
    pyfbody = loader.load(source_path).generate(Model=model, name=name)
    # 保存位置
    path = f"{output}cms/api/{str.lower(model)}.py"
    print("success created at", path)
    f = open(path, "w")
    f.write(str(pyfbody, encoding='utf-8'))
    f.close()

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-m', '--model', dest='model', type=str, default='Demo', help='target model')
    parser.add_argument('-n', '--name', dest='name', type=str, default='资源', help='target name')
    parser.add_argument('-o', '--output', dest='output', type=str, default="", help='target output folder')
    args = parser.parse_args()
    init_logic(args.model, args.output)
    init_api(args.model, args.name ,args.output)





import json
import multiprocessing
import os

workers_per_core_str = os.getenv("WORKERS_PER_CORE", "1")
web_concurrency_str = os.getenv("WEB_CONCURRENCY", None)
host = os.getenv("HOST", "0.0.0.0")
port = os.getenv("PORT", "80")
bind_env = os.getenv("BIND", None)
use_loglevel = os.getenv("LOG_LEVEL", "info")
if bind_env:
    use_bind = bind_env
else:
    use_bind = f"{host}:{port}"

cores = multiprocessing.cpu_count()
workers_per_core = float(workers_per_core_str)
default_web_concurrency = workers_per_core * cores
if web_concurrency_str:
    web_concurrency = int(web_concurrency_str)
    assert web_concurrency > 0
else:
    web_concurrency = max(int(default_web_concurrency), 2)

# Gunicorn config variables
loglevel = use_loglevel
workers = web_concurrency
bind = use_bind
keepalive = 120
errorlog = "-"
timeout = 0

# For debugging and testing
log_data = {
    "loglevel": loglevel,
    "workers": workers,
    "bind": bind,
    # Additional, non-gunicorn variables
    "workers_per_core": workers_per_core,
    "host": host,
    "port": port,
}
print(json.dumps(log_data))



from pydantic import BaseModel, Field
from typing import List
from peewee import CharField, ForeignKeyField, CompositeKey
from cms.core.db import DataViewsionManage, DBBaseModel, pgdb
from cms.logic.common.crud import CRUDBase
from cms.core.logger import Logger

logger = Logger().getLogger()


class AreaCode(DBBaseModel):
    """ 地区组织, 嵌套结构 """
    name = CharField()
    parent = ForeignKeyField('self', null=True, backref='children')


class AreaCodeBase(BaseModel):
    name: str = Field(None, max_length=40, min_length=2, title='地区名称')
    parent_id: int = Field(None, title="父节点ID")


class AreaCodeCreate(AreaCodeBase):
    pass


class AreaCodetUpdate(AreaCodeBase):
    id: int = Field(..., title="分组id")


class AreaCodeOut(AreaCodetUpdate):
    parent: AreaCodetUpdate = Field(None, title="父节点")


class CRUDAreaCode(CRUDBase[AreaCode, AreaCodeCreate, AreaCodetUpdate, AreaCodeOut]):
    
    async def get_multi(
        self, pgdvm: DataViewsionManage, *, parent_id=0,skip: int = 0, limit: int = 100, **kwargs
    ) -> List[AreaCodeOut]:
        query = self.model.select().where(AreaCode.parent==parent_id).offset(skip).limit(limit)
        lst = await pgdvm.execute(query)
        return lst

crud_AreaCode = CRUDAreaCode(AreaCode)



import sys
import asyncio
import signal
import peewee

from fastapi import FastAPI
from starlette.middleware.cors import CORSMiddleware
from starlette.requests import Request
from cms.core.error_handler import ErrorHandler
from cms.app import api_router
from cms.core import config
from cms.core.db import get_session
from cms.core.logger import Logger
from cms.logic.register import register


logger = Logger().getLogger()

app = FastAPI(title=config.PROJECT_NAME, openapi_url="/api/v1/openapi.json")

# CORS
origins = ["http://localhost:8081"]



async def shutdown(signal: signal):
    """
    彻底关闭服务, 释放端口资源
    """
    logger.info("Received exit signal %s...", signal.name)
    tasks = [t for t in asyncio.all_tasks() if t is not asyncio.current_task()]
    [task.cancel() for task in tasks]
    logger.info("Canceling outstanding tasks")
    await asyncio.gather(*tasks)



# Set all CORS enabled origins
if config.BACKEND_CORS_ORIGINS:
    origins_raw = config.BACKEND_CORS_ORIGINS.split(",")
    for origin in origins_raw:
        use_origin = origin.strip()
        origins.append(use_origin)
    app.add_middleware(
        CORSMiddleware,
        allow_origins=origins,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    ),

app.include_router(api_router, prefix=config.API_V1_STR)


@app.middleware("http")
async def db_session_middleware(request: Request, call_next):
    pgdb = get_session()
    request.state.pgdb = pgdb
    try:
        if pgdb.database.is_closed():
            logger.info('DB connecting..')
            pgdb.database.connect()

        response = await call_next(request)
        pgdb.database.close()
        return response
    except peewee.OperationalError:
        logger.info('DB reconnecting..')
        pgdb.database.connect()
        response = await call_next(request)
        return response
    except peewee.InterfaceError:
        logger.info("DB Interface Error")
        pass
    except Exception as e:
        ttype, tvalue, ttraceback = sys.exc_info()
        return ErrorHandler.handler(e, ttype, tvalue, ttraceback)



@app.on_event("startup")
async def startup():
    logger.info('startup')
    loop = asyncio.get_event_loop()
    signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
    for s in signals:
        loop.add_signal_handler(s, lambda s=s: asyncio.create_task(shutdown(s)))

    register()
    logger.info('startup end')



# if __name__ == '__main__' and config.DEBUG:
#     import uvicorn
#     uvicorn.run(app, host='0.0.0.0', port=80, debug=True)



import requests

from cms.core import config
from cms.core.logger import logger


def register():
    modules = {
        "dataCheck": "测试功能",
    }

    resources = {
        "resource_project": "测试资源"
    }

    val = {
        "modules": modules,
        "resources": resources,
        "tenant_id": "1",
        "client_id": config.CLIENT_ID,
        "client_secret": config.CLIENT_SECRET,
        "is_saas": True
    }
    try:
        r = requests.post(config.USERCENTER_HOST + "/api/v1/services/attr",
                          json=val)
        if len(r.json().get("modules", {})) == 0:
            logger.info(r.json())
    except Exception as e:
        logger.error(f"注册资源模块失败host:{config.USERCENTER_HOST},error:{e}")



"""
    author comger@gmail.com
"""
import jwt
from datetime import datetime, timedelta
from passlib.context import CryptContext
from fastapi import HTTPException, Security
from fastapi.security import OAuth2PasswordBearer
from jwt import PyJWTError
from starlette.status import HTTP_403_FORBIDDEN
from cms.core import config

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
reusable_oauth2 = OAuth2PasswordBearer(tokenUrl=config.LOGIN_API)


def verify_password(plain_password: str, hashed_password: str):
    # return hashlib.md5(password).hexdigest() == hashed_password
    try:
        return pwd_context.verify(plain_password, hashed_password)
    except Exception:
        return False


def get_password_hash(password: str):
    # print(hashlib.md5(password.encode('utf-8')).hexdigest())
    return pwd_context.hash(password)


def create_access_token(*, data: dict, expires_delta: timedelta = None):
    """ 创建Token, 内容为data """
    to_encode = data.copy()
    if not expires_delta:
        expires_delta = timedelta(minutes=config.ACCESS_TOKEN_EXPIRE_MINUTES)

    expire = datetime.utcnow() + expires_delta
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(
        to_encode, config.SECRET_KEY, algorithm=config.ALGORITHM)
    return encoded_jwt


def get_current_user(token: str = Security(reusable_oauth2)):
    """ 解码Token 获取明文信息"""
    try:
        payload = jwt.decode(token, config.SECRET_KEY, algorithms=[config.ALGORITHM])
        return payload
    except PyJWTError:
        raise HTTPException(
            status_code=HTTP_403_FORBIDDEN, detail="验证Token失败"
        )


def get_current_active_user(current_user: Security(get_current_user)):
    if "is_active" in current_user and not current_user['is_active']:
        raise HTTPException(status_code=403, detail="用户已被禁用")
    return current_user


def get_current_active_superuser(current_user: Security(get_current_active_user)):
    if "is_superuser" in current_user and not current_user["is_superuser"]:
        raise HTTPException(
            status_code=403, detail="用户不是超管"
        )
    return current_user



from typing import List
from fastapi import APIRouter, Depends, Query, Path

from cms.core.auth import get_current_user
from cms.core.db import depends_db, m2d
from cms.logic.areacode import crud_AreaCode, AreaCodeCreate, AreaCodetUpdate, AreaCodeOut


router = APIRouter()



@router.get('/areacodes/{parent_id}', summary='按父ID获取信息', response_model=List[AreaCodeOut])
async def get_list(parent_id: int = Path(..., title="parent_id"),
                    pgdb=Depends(depends_db),
                    current_user=Depends(get_current_user)):
    """
    按ID获取信息
    """
    lst = await crud_AreaCode.get_multi(pgdvm=pgdb, parent_id=parent_id)
    return lst


@router.post("/areacodes", summary='添加', status_code=201, response_model=AreaCodeOut)
async def create(obj_in: AreaCodeCreate,
                       pgdb=Depends(depends_db),
                       current_user=Depends(get_current_user)):
    """
        添加
    """
    obj = await crud_AreaCode.create(pgdvm=pgdb, obj_in=obj_in)
    return m2d(obj)



@router.delete('/areacodes/{id}', summary='按ID删除信息')
async def delete(id: int = Path(..., title="ID"),
                    pgdb=Depends(depends_db),
                    current_user=Depends(get_current_user)):
    """
    按ID删除信息, 如果有下级或被关联,则无法删除
    """
    obj = await crud_AreaCode.remove(pgdvm=pgdb, id=id)
    return m2d(obj)



#!/usr/bin/env python
#
# Copyright 2009 comger@gmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from setuptools import setup, find_packages

kwargs = {}


def _process_requirements():
    packages = open('requirements.txt').read().strip().split('\n')
    requires = []
    for pkg in packages:
        if pkg.startswith('git+ssh'):
            return_code = os.system('pip install {}'.format(pkg))
            assert return_code == 0, 'error, status_code is: {}, exit!'.format(return_code)
        else:
            requires.append(pkg)
    return requires



setup(name="fastapicli",
      version="0.0.2",
      packages=find_packages(where='.', exclude=(), include=('*',)),
      package_data={'': ['*.*']},
      exclude_package_data={'': ["*.pyc"]},
      author="comger@gmail.com",
      author_email="comger@gmail.com",
      url="http://github.com/comger/fastapicli",
      license="http://www.apache.org/licenses/LICENSE-2.0",
      description="fastapicli is a easy tool for fastapi orm with jwt",
      scripts=['fastapicli/fastapi_init.py','fastapicli/fastapi_api.py'],
      classifiers=[
          'License :: OSI Approved :: Apache Software License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
      ],
      keywords=["fastapi", "orm", "peewee", "jwt", "codegen"],
      install_requires=["tornado"],
      **kwargs)



import uuid
from peewee import CharField, ForeignKeyField, BooleanField
from playhouse.postgres_ext import HStoreField, ArrayField
from typing import Optional, List
from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel, EmailStr, Field
from cms.core.config import FIRST_USER, FIRST_USER_PASSWORD
from cms.core.db import DataViewsionManage, DBBaseModel
from cms.core.auth import verify_password, get_password_hash
from cms.logic.common.crud import CRUDBase, ModelType


class UserRole(DBBaseModel):
    name = CharField(unique=True)
    scopes = ArrayField(CharField, null=True, default=[])


class RoleBase(BaseModel):
    name: str = Field(None,
                      title="角色名称",
                      description="名称不可为空, 字符长度范围2~40",
                      max_length=40,
                      min_length=2)

    scopes: Optional[List[str]] = Field([], title="权限节点列表")


class RoleCreate(RoleBase):
    pass


class RoleUpdate(RoleBase):
    pass


class RoleOut(RoleBase):
    id: int = Field(..., title="ID")


class CRUDRole(CRUDBase[UserRole, RoleCreate, RoleUpdate, RoleOut]):
    pass


class User(DBBaseModel):
    uid = CharField(default=str(uuid.uuid1()), index=True, unique=True)
    username = CharField(unique=True)
    nickname = CharField(null=True, default="")
    email = CharField(unique=True, default="")
    password = CharField()
    tel = CharField(null=True, default="")
    is_superuser = BooleanField(default=False, null=True)
    is_active = BooleanField(default=True, null=True)
    role = ForeignKeyField(UserRole, backref='user', null=True)
    attrs = HStoreField(default={})


class UserBase(BaseModel):
    nickname: str = Field(None,
                          title="昵称",
                          description="昵称不可为空, 字符长度范围2~40",
                          max_length=40,
                          min_length=2)
    email: EmailStr = Field(None,
                            title="邮箱",
                            description="邮箱不可为空, 具备唯一性, 字符长度范围6~40")
    tel: Optional[str] = Field(None, title="手机号", description="长度必须为11. 允许为空.")
    is_superuser: Optional[bool] = Field(0,
                                         title="是否为超管",
                                         description="此处为用户中心超管, 0:非超管, 1: 超管")
    is_active: Optional[bool] = Field(1,
                                      title="是否禁用",
                                      description="禁用后,用户不能正常登录,0： 禁用, 1: 启用")
    role_id: Optional[int] = Field(None, title="角色id", description="角色外键")


class UserUpdate(UserBase):
    pass


class UserUpdateMe(UserBase):
    pass


class UserCreate(UserBase):
    username: str = Field(...,
                          title="用户名",
                          description="用户名不可为空, 具备唯一性, 字符长度范围6~40",
                          max_length=40,
                          min_length=6)
    password: str = Field(...,
                          title="密码",
                          description="密码不可为空, 字符长度范围6~40",
                          max_length=40,
                          min_length=6)
    nickname: str = Field(...,
                          title="昵称",
                          description="昵称不可为空, 字符长度范围2~40",
                          max_length=40,
                          min_length=2)
    email: EmailStr = Field(...,
                            title="邮箱",
                            description="邮箱不可为空, 具备唯一性, 字符长度范围6~40")


class UserOut(UserUpdate):
    id: int = Field(..., title="ID")
    username: str = Field(...,
                          title="用户名",
                          description="用户名不可为空, 具备唯一性, 字符长度范围6~40",
                          max_length=40,
                          min_length=6)
    role: RoleOut = Field(None, title="角色信息")


class Token(BaseModel):
    access_token: str
    token_type: str


class CRUDUser(CRUDBase[User, UserCreate, UserUpdate, UserOut]):
    async def create(self, pgdvm: DataViewsionManage, *,
                     obj_in: UserCreate) -> User:
        obj_in_data = jsonable_encoder(obj_in)
        obj_in_data["password"] = get_password_hash(obj_in.password)
        obj = await pgdvm.create(self.model, **obj_in_data)
        return obj

    async def login(self, pgdvm: DataViewsionManage, username: str,
                    password: str):
        """
        用户登录验证
        返回
            true, user
            false, message
        """

        try:
            user = await pgdvm.get(User, username=username)
            if not verify_password(password, user.password):
                return False, "密码错误"
            if not user.is_active:
                return False, "用户已被禁用"

            return True, user
        except User.DoesNotExist:
            return False, "用户名不存在"

    async def get_multi(self,
                        pgdvm: DataViewsionManage,
                        *,
                        skip: int = 0,
                        limit: int = 100,
                        ref: bool = True,
                        **kwargs) -> List[ModelType]:
        """
        支持查询参数query key , 可匹配 username, nickname, email, tel
        """
        querykey = kwargs.get("querykey", None)
        query = self.model.select()
        if querykey:
            query_where = (User.username**f"%{querykey}%") | (
                User.email**f"%{querykey}%") | (User.tel**f"%{querykey}%") | (
                    User.nickname**f"%{querykey}%")
            query = query.where(query_where)

        query = query.order_by(User.id.desc()).offset(skip).limit(limit)
        # if not ref:
        #     query = query.dicts()
        lst = await pgdvm.execute(query)
        return lst


def init_super_user():
    """
    初始化超管
    """
    try:
        User.create(**{
            "username": FIRST_USER,
            "nickname": 'admin',
            "email": FIRST_USER,
            "password": get_password_hash(FIRST_USER_PASSWORD),
            "is_superuser": True,
            "is_active": True
        })
    except Exception as e:
        print("Has Init First User", e)


crud_user = CRUDUser(User)
crud_role = CRUDRole(UserRole)



"""
FastApiCli
author comger@gmail.com

fastapi_init.py projectname
"""
import sys
import os
import shutil
import fastapicli


def init_project(to_path="fastapi_demo"):
    """ 初始化项目 """
    root = fastapicli.__path__[0]
    from_path = root + '/fastapidemo'
    to_path = f"{os.getcwd()}/{path}"
    shutil.copytree(from_path, to_path)
    print(f"success init {path}")
    

if __name__ == '__main__':
    if len(sys.argv)>1:
        path = sys.argv[1]
        init_project(path)



    
